@@ -1,124 +1,47 @@
 // This file is part of the uutils coreutils package.
 //
-// (c) Rolf Morel <rolfmorel@gmail.com>
-//
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
 // spell-checker:ignore (ToDO) delim sourcefiles
 
-#[macro_use]
-extern crate uucore;
-
 use bstr::io::BufReadExt;
-use clap::{crate_version, Arg, Command};
+use clap::{crate_version, Arg, ArgAction, Command};
+use is_terminal::IsTerminal;
 use std::fs::File;
 use std::io::{stdin, stdout, BufReader, BufWriter, Read, Write};
 use std::path::Path;
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UResult, USimpleError};
+use uucore::line_ending::LineEnding;
 
 use self::searcher::Searcher;
-use uucore::format_usage;
+use matcher::{ExactMatcher, Matcher, WhitespaceMatcher};
 use uucore::ranges::Range;
+use uucore::{format_usage, help_about, help_section, help_usage, show, show_error, show_if_err};
 
+mod matcher;
 mod searcher;
 
-static NAME: &str = "cut";
-static USAGE: &str =
-    "{} [-d] [-s] [-z] [--output-delimiter] ((-f|-b|-c) {{sequence}}) {{sourcefile}}+";
-static ABOUT: &str =
-    "Prints specified byte or field columns from each line of stdin or the input files";
-static LONG_HELP: &str = "
- Each call must specify a mode (what to use for columns),
- a sequence (which columns to print), and provide a data source
-
- Specifying a mode
-
-    Use --bytes (-b) or --characters (-c) to specify byte mode
-
-    Use --fields (-f) to specify field mode, where each line is broken into
-    fields identified by a delimiter character. For example for a typical CSV
-    you could use this in combination with setting comma as the delimiter
-
- Specifying a sequence
-
-    A sequence is a group of 1 or more numbers or inclusive ranges separated
-    by a commas.
-
-    cut -f 2,5-7 some_file.txt
-    will display the 2nd, 5th, 6th, and 7th field for each source line
-
-    Ranges can extend to the end of the row by excluding the the second number
-
-    cut -f 3- some_file.txt
-    will display the 3rd field and all fields after for each source line
-
-    The first number of a range can be excluded, and this is effectively the
-    same as using 1 as the first number: it causes the range to begin at the
-    first column. Ranges can also display a single column
-
-    cut -f 1,3-5 some_file.txt
-    will display the 1st, 3rd, 4th, and 5th field for each source line
-
-    The --complement option, when used, inverts the effect of the sequence
-
-    cut --complement -f 4-6 some_file.txt
-    will display the every field but the 4th, 5th, and 6th
-
- Specifying a data source
-
-    If no sourcefile arguments are specified, stdin is used as the source of
-    lines to print
-
-    If sourcefile arguments are specified, stdin is ignored and all files are
-    read in consecutively if a sourcefile is not successfully read, a warning
-    will print to stderr, and the eventual status code will be 1, but cut
-    will continue to read through proceeding sourcefiles
-
-    To print columns from both STDIN and a file argument, use - (dash) as a
-    sourcefile argument to represent stdin.
-
- Field Mode options
-
-    The fields in each line are identified by a delimiter (separator)
-
-    Set the delimiter
-        Set the delimiter which separates fields in the file using the
-        --delimiter (-d) option. Setting the delimiter is optional.
-        If not set, a default delimiter of Tab will be used.
-
-    Optionally Filter based on delimiter
-        If the --only-delimited (-s) flag is provided, only lines which
-        contain the delimiter will be printed
-
-    Replace the delimiter
-        If the --output-delimiter option is provided, the argument used for
-        it will replace the delimiter character in each line printed. This is
-        useful for transforming tabular data - e.g. to convert a CSV to a
-        TSV (tab-separated file)
-
- Line endings
-
-    When the --zero-terminated (-z) option is used, cut sees \\0 (null) as the
-    'line ending' character (both for the purposes of reading lines and
-    separating printed lines) instead of \\n (newline). This is useful for
-    tabular data where some of the cells may contain newlines
-
-    echo 'ab\\0cd' | cut -z -c 1
-    will result in 'a\\0c\\0'
-";
+const USAGE: &str = help_usage!("cut.md");
+const ABOUT: &str = help_about!("cut.md");
+const AFTER_HELP: &str = help_section!("after help", "cut.md");
 
 struct Options {
     out_delim: Option<String>,
-    zero_terminated: bool,
+    line_ending: LineEnding,
+}
+
+enum Delimiter {
+    Whitespace,
+    String(String), // FIXME: use char?
 }
 
 struct FieldOptions {
-    delimiter: String, // one char long, String because of UTF8 representation
+    delimiter: Delimiter,
     out_delimiter: Option<String>,
     only_delimited: bool,
-    zero_terminated: bool,
+    line_ending: LineEnding,
 }
 
 enum Mode {
@@ -128,7 +51,7 @@ enum Mode {
 }
 
 fn stdout_writer() -> Box<dyn Write> {
-    if atty::is(atty::Stream::Stdout) {
+    if std::io::stdout().is_terminal() {
         Box::new(stdout())
     } else {
         Box::new(BufWriter::new(stdout())) as Box<dyn Write>
@@ -144,8 +67,8 @@ fn list_to_ranges(list: &str, complement: bool) -> Result<Vec<Range>, String> {
 }
 
 fn cut_bytes<R: Read>(reader: R, ranges: &[Range], opts: &Options) -> UResult<()> {
-    let newline_char = if opts.zero_terminated { b'\0' } else { b'\n' };
-    let buf_in = BufReader::new(reader);
+    let newline_char = opts.line_ending.into();
+    let mut buf_in = BufReader::new(reader);
     let mut out = stdout_writer();
     let delim = opts
         .out_delim
@@ -180,23 +103,22 @@ fn cut_bytes<R: Read>(reader: R, ranges: &[Range], opts: &Options) -> UResult<()
     Ok(())
 }
 
-#[allow(clippy::cognitive_complexity)]
-fn cut_fields_delimiter<R: Read>(
+// Output delimiter is explicitly specified
+fn cut_fields_explicit_out_delim<R: Read, M: Matcher>(
     reader: R,
+    matcher: &M,
     ranges: &[Range],
-    delim: &str,
     only_delimited: bool,
     newline_char: u8,
     out_delim: &str,
 ) -> UResult<()> {
-    let buf_in = BufReader::new(reader);
+    let mut buf_in = BufReader::new(reader);
     let mut out = stdout_writer();
-    let input_delim_len = delim.len();
 
     let result = buf_in.for_byte_record_with_terminator(newline_char, |line| {
         let mut fields_pos = 1;
         let mut low_idx = 0;
-        let mut delim_search = Searcher::new(line, delim.as_bytes()).peekable();
+        let mut delim_search = Searcher::new(matcher, line).peekable();
         let mut print_delim = false;
 
         if delim_search.peek().is_none() {
@@ -212,13 +134,17 @@ fn cut_fields_delimiter<R: Read>(
 
         for &Range { low, high } in ranges {
             if low - fields_pos > 0 {
+                // current field is not in the range, so jump to the field corresponding to the
+                // beginning of the range if any
                 low_idx = match delim_search.nth(low - fields_pos - 1) {
-                    Some(index) => index + input_delim_len,
+                    Some((_, last)) => last,
                     None => break,
                 };
             }
 
+            // at this point, current field is the first in the range
             for _ in 0..=high - low {
+                // skip printing delimiter if this is the first matching field for this line
                 if print_delim {
                     out.write_all(out_delim.as_bytes())?;
                 } else {
@@ -226,15 +152,17 @@ fn cut_fields_delimiter<R: Read>(
                 }
 
                 match delim_search.next() {
-                    Some(high_idx) => {
-                        let segment = &line[low_idx..high_idx];
+                    // print the current field up to the next field delim
+                    Some((first, last)) => {
+                        let segment = &line[low_idx..first];
 
                         out.write_all(segment)?;
 
-                        low_idx = high_idx + input_delim_len;
+                        low_idx = last;
                         fields_pos = high + 1;
                     }
                     None => {
+                        // this is the last field in the line, so print the rest
                         let segment = &line[low_idx..];
 
                         out.write_all(segment)?;
@@ -259,32 +187,25 @@ fn cut_fields_delimiter<R: Read>(
     Ok(())
 }
 
-#[allow(clippy::cognitive_complexity)]
-fn cut_fields<R: Read>(reader: R, ranges: &[Range], opts: &FieldOptions) -> UResult<()> {
-    let newline_char = if opts.zero_terminated { b'\0' } else { b'\n' };
-    if let Some(ref o_delim) = opts.out_delimiter {
-        return cut_fields_delimiter(
-            reader,
-            ranges,
-            &opts.delimiter,
-            opts.only_delimited,
-            newline_char,
-            o_delim,
-        );
-    }
-
-    let buf_in = BufReader::new(reader);
+// Output delimiter is the same as input delimiter
+fn cut_fields_implicit_out_delim<R: Read, M: Matcher>(
+    reader: R,
+    matcher: &M,
+    ranges: &[Range],
+    only_delimited: bool,
+    newline_char: u8,
+) -> UResult<()> {
+    let mut buf_in = BufReader::new(reader);
     let mut out = stdout_writer();
-    let delim_len = opts.delimiter.len();
 
     let result = buf_in.for_byte_record_with_terminator(newline_char, |line| {
         let mut fields_pos = 1;
         let mut low_idx = 0;
-        let mut delim_search = Searcher::new(line, opts.delimiter.as_bytes()).peekable();
+        let mut delim_search = Searcher::new(matcher, line).peekable();
         let mut print_delim = false;
 
         if delim_search.peek().is_none() {
-            if !opts.only_delimited {
+            if !only_delimited {
                 out.write_all(line)?;
                 if line[line.len() - 1] != newline_char {
                     out.write_all(&[newline_char])?;
@@ -296,25 +217,21 @@ fn cut_fields<R: Read>(reader: R, ranges: &[Range], opts: &FieldOptions) -> URes
 
         for &Range { low, high } in ranges {
             if low - fields_pos > 0 {
-                if let Some(delim_pos) = delim_search.nth(low - fields_pos - 1) {
-                    low_idx = if print_delim {
-                        delim_pos
-                    } else {
-                        delim_pos + delim_len
-                    }
+                if let Some((first, last)) = delim_search.nth(low - fields_pos - 1) {
+                    low_idx = if print_delim { first } else { last }
                 } else {
                     break;
                 }
             }
 
             match delim_search.nth(high - low) {
-                Some(high_idx) => {
-                    let segment = &line[low_idx..high_idx];
+                Some((first, _)) => {
+                    let segment = &line[low_idx..first];
 
                     out.write_all(segment)?;
 
                     print_delim = true;
-                    low_idx = high_idx;
+                    low_idx = first;
                     fields_pos = high + 1;
                 }
                 None => {
@@ -340,7 +257,45 @@ fn cut_fields<R: Read>(reader: R, ranges: &[Range], opts: &FieldOptions) -> URes
     Ok(())
 }
 
-fn cut_files(mut filenames: Vec<String>, mode: &Mode) -> UResult<()> {
+fn cut_fields<R: Read>(reader: R, ranges: &[Range], opts: &FieldOptions) -> UResult<()> {
+    let newline_char = opts.line_ending.into();
+    match opts.delimiter {
+        Delimiter::String(ref delim) => {
+            let matcher = ExactMatcher::new(delim.as_bytes());
+            match opts.out_delimiter {
+                Some(ref out_delim) => cut_fields_explicit_out_delim(
+                    reader,
+                    &matcher,
+                    ranges,
+                    opts.only_delimited,
+                    newline_char,
+                    out_delim,
+                ),
+                None => cut_fields_implicit_out_delim(
+                    reader,
+                    &matcher,
+                    ranges,
+                    opts.only_delimited,
+                    newline_char,
+                ),
+            }
+        }
+        Delimiter::Whitespace => {
+            let matcher = WhitespaceMatcher {};
+            let out_delim = opts.out_delimiter.as_deref().unwrap_or("\t");
+            cut_fields_explicit_out_delim(
+                reader,
+                &matcher,
+                ranges,
+                opts.only_delimited,
+                newline_char,
+                out_delim,
+            )
+        }
+    }
+}
+
+fn cut_files(mut filenames: Vec<String>, mode: &Mode) {
     let mut stdin_read = false;
 
     if filenames.is_empty() {
@@ -368,7 +323,7 @@ fn cut_files(mut filenames: Vec<String>, mode: &Mode) -> UResult<()> {
                 continue;
             }
 
-            show_if_err!(File::open(&path)
+            show_if_err!(File::open(path)
                 .map_err_context(|| filename.maybe_quote().to_string())
                 .and_then(|file| {
                     match &mode {
@@ -380,8 +335,6 @@ fn cut_files(mut filenames: Vec<String>, mode: &Mode) -> UResult<()> {
                 }));
         }
     }
-
-    Ok(())
 }
 
 mod options {
@@ -392,6 +345,7 @@ mod options {
     pub const ZERO_TERMINATED: &str = "zero-terminated";
     pub const ONLY_DELIMITED: &str = "only-delimited";
     pub const OUTPUT_DELIMITER: &str = "output-delimiter";
+    pub const WHITESPACE_DELIMITED: &str = "whitespace-delimited";
     pub const COMPLEMENT: &str = "complement";
     pub const FILE: &str = "file";
 }
@@ -403,7 +357,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let delimiter_is_equal = args.contains(&"-d=".to_string()); // special case
     let matches = uu_app().try_get_matches_from(args)?;
 
-    let complement = matches.contains_id(options::COMPLEMENT);
+    let complement = matches.get_flag(options::COMPLEMENT);
 
     let mode_parse = match (
         matches.get_one::<String>(options::BYTES),
@@ -421,7 +375,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                             .unwrap_or_default()
                             .to_owned(),
                     ),
-                    zero_terminated: matches.contains_id(options::ZERO_TERMINATED),
+                    line_ending: LineEnding::from_zero_flag(matches.get_flag(options::ZERO_TERMINATED)),
                 },
             )
         }),
@@ -436,7 +390,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                             .unwrap_or_default()
                             .to_owned(),
                     ),
-                    zero_terminated: matches.contains_id(options::ZERO_TERMINATED),
+                    line_ending: LineEnding::from_zero_flag(matches.get_flag(options::ZERO_TERMINATED)),
                 },
             )
         }),
@@ -453,10 +407,15 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                     None => None,
                 };
 
-                let only_delimited = matches.contains_id(options::ONLY_DELIMITED);
-                let zero_terminated = matches.contains_id(options::ZERO_TERMINATED);
+                let only_delimited = matches.get_flag(options::ONLY_DELIMITED);
+                let whitespace_delimited = matches.get_flag(options::WHITESPACE_DELIMITED);
+                let zero_terminated = matches.get_flag(options::ZERO_TERMINATED);
+                let line_ending = LineEnding::from_zero_flag(zero_terminated);
 
                 match matches.get_one::<String>(options::DELIMITER).map(|s| s.as_str()) {
+                    Some(_) if whitespace_delimited => {
+                            Err("invalid input: Only one of --delimiter (-d) or -w option can be specified".into())
+                        }
                     Some(mut delim) => {
                         // GNU's `cut` supports `-d=` to set the delimiter to `=`.
                         // Clap parsing is limited in this situation, see:
@@ -479,10 +438,10 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                             Ok(Mode::Fields(
                                 ranges,
                                 FieldOptions {
-                                    delimiter: delim,
+                                    delimiter: Delimiter::String(delim),
                                     out_delimiter: out_delim,
                                     only_delimited,
-                                    zero_terminated,
+                                    line_ending,
                                 },
                             ))
                         }
@@ -490,10 +449,13 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                     None => Ok(Mode::Fields(
                         ranges,
                         FieldOptions {
-                            delimiter: "\t".to_owned(),
+                            delimiter: match whitespace_delimited {
+                                true => Delimiter::Whitespace,
+                                false => Delimiter::String("\t".to_owned()),
+                            },
                             out_delimiter: out_delim,
                             only_delimited,
-                            zero_terminated,
+                            line_ending,
                         },
                     )),
                 }
@@ -514,7 +476,12 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                 Err("invalid input: The '--delimiter' ('-d') option only usable if printing a sequence of fields".into())
             }
             Mode::Bytes(_, _) | Mode::Characters(_, _)
-                if matches.contains_id(options::ONLY_DELIMITED) =>
+                if matches.get_flag(options::WHITESPACE_DELIMITED) =>
+            {
+                Err("invalid input: The '-w' option only usable if printing a sequence of fields".into())
+            }
+            Mode::Bytes(_, _) | Mode::Characters(_, _)
+                if matches.get_flag(options::ONLY_DELIMITED) =>
             {
                 Err("invalid input: The '--only-delimited' ('-s') option only usable if printing a sequence of fields".into())
             }
@@ -529,93 +496,89 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         .collect();
 
     match mode_parse {
-        Ok(mode) => cut_files(files, &mode),
+        Ok(mode) => {
+            cut_files(files, &mode);
+            Ok(())
+        }
         Err(e) => Err(USimpleError::new(1, e)),
     }
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .name(NAME)
         .version(crate_version!())
         .override_usage(format_usage(USAGE))
         .about(ABOUT)
-        .after_help(LONG_HELP)
+        .after_help(AFTER_HELP)
         .infer_long_args(true)
         .arg(
             Arg::new(options::BYTES)
                 .short('b')
                 .long(options::BYTES)
-                .takes_value(true)
                 .help("filter byte columns from the input source")
                 .allow_hyphen_values(true)
-                .value_name("LIST")
-                .display_order(1),
+                .value_name("LIST"),
         )
         .arg(
             Arg::new(options::CHARACTERS)
                 .short('c')
                 .long(options::CHARACTERS)
                 .help("alias for character mode")
-                .takes_value(true)
                 .allow_hyphen_values(true)
-                .value_name("LIST")
-                .display_order(2),
+                .value_name("LIST"),
         )
         .arg(
             Arg::new(options::DELIMITER)
                 .short('d')
                 .long(options::DELIMITER)
                 .help("specify the delimiter character that separates fields in the input source. Defaults to Tab.")
-                .takes_value(true)
-                .value_name("DELIM")
-                .display_order(3),
+                .value_name("DELIM"),
+        )
+        .arg(
+            Arg::new(options::WHITESPACE_DELIMITED)
+                .short('w')
+                .help("Use any number of whitespace (Space, Tab) to separate fields in the input source (FreeBSD extension).")
+                .value_name("WHITESPACE")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FIELDS)
                 .short('f')
                 .long(options::FIELDS)
                 .help("filter field columns from the input source")
-                .takes_value(true)
                 .allow_hyphen_values(true)
-                .value_name("LIST")
-                .display_order(4),
+                .value_name("LIST"),
         )
         .arg(
             Arg::new(options::COMPLEMENT)
                 .long(options::COMPLEMENT)
                 .help("invert the filter - instead of displaying only the filtered columns, display all but those columns")
-                .takes_value(false)
-                .display_order(5),
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::ONLY_DELIMITED)
-            .short('s')
+                .short('s')
                 .long(options::ONLY_DELIMITED)
                 .help("in field mode, only print lines which contain the delimiter")
-                .takes_value(false)
-                .display_order(6),
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::ZERO_TERMINATED)
-            .short('z')
+                .short('z')
                 .long(options::ZERO_TERMINATED)
                 .help("instead of filtering columns based on line, filter columns based on \\0 (NULL character)")
-                .takes_value(false)
-                .display_order(8),
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OUTPUT_DELIMITER)
-            .long(options::OUTPUT_DELIMITER)
+                .long(options::OUTPUT_DELIMITER)
                 .help("in field mode, replace the delimiter in output lines with this option's argument")
-                .takes_value(true)
-                .value_name("NEW_DELIM")
-                .display_order(7),
+                .value_name("NEW_DELIM"),
         )
         .arg(
             Arg::new(options::FILE)
             .hide(true)
-            .multiple_occurrences(true)
+            .action(ArgAction::Append)
             .value_hint(clap::ValueHint::FilePath)
         )
 }
