@@ -1,15 +1,12 @@
 // This file is part of the uutils coreutils package.
 //
-// (c) Anthony Deschamps <anthony.j.deschamps@gmail.com>
-// (c) Sylvestre Ledru <sylvestre@debian.org>
-//
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-// spell-checker:ignore (chrono) Datelike Timelike ; (format) DATEFILE MMDDhhmm ; (vars) datetime datetimes humantime
+// spell-checker:ignore (chrono) Datelike Timelike ; (format) DATEFILE MMDDhhmm ; (vars) datetime datetimes
 
 use chrono::format::{Item, StrftimeItems};
-use chrono::{DateTime, Duration as ChronoDuration, FixedOffset, Local, Offset, Utc};
+use chrono::{DateTime, Duration, FixedOffset, Local, Offset, Utc};
 #[cfg(windows)]
 use chrono::{Datelike, Timelike};
 use clap::{crate_version, Arg, ArgAction, Command};
@@ -18,7 +15,6 @@ use libc::{clock_settime, timespec, CLOCK_REALTIME};
 use std::fs::File;
 use std::io::{BufRead, BufReader};
 use std::path::PathBuf;
-use time::Duration;
 use uucore::display::Quotable;
 #[cfg(not(any(target_os = "redox")))]
 use uucore::error::FromIo;
@@ -27,14 +23,13 @@ use uucore::{format_usage, help_about, help_usage, show};
 #[cfg(windows)]
 use windows_sys::Win32::{Foundation::SYSTEMTIME, System::SystemInformation::SetSystemTime};
 
+use uucore::shortcut_value_parser::ShortcutValueParser;
+
 // Options
 const DATE: &str = "date";
 const HOURS: &str = "hours";
 const MINUTES: &str = "minutes";
 const SECONDS: &str = "seconds";
-const HOUR: &str = "hour";
-const MINUTE: &str = "minute";
-const SECOND: &str = "second";
 const NS: &str = "ns";
 
 const ABOUT: &str = help_about!("date.md");
@@ -111,9 +106,9 @@ enum Iso8601Format {
 impl<'a> From<&'a str> for Iso8601Format {
     fn from(s: &str) -> Self {
         match s {
-            HOURS | HOUR => Self::Hours,
-            MINUTES | MINUTE => Self::Minutes,
-            SECONDS | SECOND => Self::Seconds,
+            HOURS => Self::Hours,
+            MINUTES => Self::Minutes,
+            SECONDS => Self::Seconds,
             NS => Self::Ns,
             DATE => Self::Date,
             // Note: This is caught by clap via `possible_values`
@@ -132,7 +127,7 @@ impl<'a> From<&'a str> for Rfc3339Format {
     fn from(s: &str) -> Self {
         match s {
             DATE => Self::Date,
-            SECONDS | SECOND => Self::Seconds,
+            SECONDS => Self::Seconds,
             NS => Self::Ns,
             // Should be caught by clap
             _ => panic!("Invalid format: {s}"),
@@ -171,7 +166,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     };
 
     let date_source = if let Some(date) = matches.get_one::<String>(OPT_DATE) {
-        if let Ok(duration) = humantime_to_duration::from_str(date.as_str()) {
+        if let Ok(duration) = parse_datetime::from_str(date.as_str()) {
             DateSource::Human(duration)
         } else {
             DateSource::Custom(date.into())
@@ -226,14 +221,23 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                 let iter = std::iter::once(date);
                 Box::new(iter)
             }
-            DateSource::Human(ref input) => {
-                // Get the current DateTime<FixedOffset> and convert the input time::Duration to chrono::Duration
-                // for things like "1 year ago"
+            DateSource::Human(relative_time) => {
+                // Get the current DateTime<FixedOffset> for things like "1 year ago"
                 let current_time = DateTime::<FixedOffset>::from(Local::now());
-                let input_chrono = ChronoDuration::seconds(input.as_seconds_f32() as i64)
-                    + ChronoDuration::nanoseconds(input.subsec_nanoseconds() as i64);
-                let iter = std::iter::once(Ok(current_time + input_chrono));
-                Box::new(iter)
+                // double check the result is overflow or not of the current_time + relative_time
+                // it may cause a panic of chrono::datetime::DateTime add
+                match current_time.checked_add_signed(relative_time) {
+                    Some(date) => {
+                        let iter = std::iter::once(Ok(date));
+                        Box::new(iter)
+                    }
+                    None => {
+                        return Err(USimpleError::new(
+                            1,
+                            format!("invalid date {}", relative_time),
+                        ));
+                    }
+                }
             }
             DateSource::File(ref path) => {
                 if path.is_dir() {
@@ -321,7 +325,9 @@ pub fn uu_app() -> Command {
                 .short('I')
                 .long(OPT_ISO_8601)
                 .value_name("FMT")
-                .value_parser([DATE, HOUR, HOURS, MINUTE, MINUTES, SECOND, SECONDS, NS])
+                .value_parser(ShortcutValueParser::new([
+                    DATE, HOURS, MINUTES, SECONDS, NS,
+                ]))
                 .num_args(0..=1)
                 .default_missing_value(OPT_DATE)
                 .help(ISO_8601_HELP_STRING),
@@ -337,7 +343,7 @@ pub fn uu_app() -> Command {
             Arg::new(OPT_RFC_3339)
                 .long(OPT_RFC_3339)
                 .value_name("FMT")
-                .value_parser([DATE, SECOND, SECONDS, NS])
+                .value_parser(ShortcutValueParser::new([DATE, SECONDS, NS]))
                 .help(RFC_3339_HELP_STRING),
         )
         .arg(
