@@ -1,7 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 // spell-checker:ignore tmpfs Pcent Itotal Iused Iavail Ipcent
 //! The filesystem usage data table.
 //!
@@ -432,12 +432,21 @@ impl fmt::Display for Table {
         while let Some(row) = row_iter.next() {
             let mut col_iter = row.iter().enumerate().peekable();
             while let Some((i, elem)) = col_iter.next() {
+                let is_last_col = col_iter.peek().is_none();
+
                 match self.alignments[i] {
-                    Alignment::Left => write!(f, "{:<width$}", elem, width = self.widths[i])?,
+                    Alignment::Left => {
+                        if is_last_col {
+                            // no trailing spaces in last column
+                            write!(f, "{elem}")?;
+                        } else {
+                            write!(f, "{:<width$}", elem, width = self.widths[i])?;
+                        }
+                    }
                     Alignment::Right => write!(f, "{:>width$}", elem, width = self.widths[i])?,
                 }
 
-                if col_iter.peek().is_some() {
+                if !is_last_col {
                     // column separator
                     write!(f, " ")?;
                 }
@@ -504,7 +513,7 @@ mod tests {
 
     #[test]
     fn test_default_header() {
-        let options = Default::default();
+        let options = Options::default();
         assert_eq!(
             Header::get_headers(&options),
             vec!(
