@@ -1,8 +1,5 @@
 // This file is part of the uutils coreutils package.
 //
-// (c) Alan Andrade <alan.andradec@gmail.com>
-// (c) Jian Zeng <anonymousknight96 AT gmail.com>
-//
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
@@ -36,19 +33,17 @@
 #![allow(non_camel_case_types)]
 #![allow(dead_code)]
 
-#[macro_use]
-extern crate uucore;
-
-use clap::{crate_version, Arg, Command};
+use clap::{crate_version, Arg, ArgAction, Command};
 use std::ffi::CStr;
 use uucore::display::Quotable;
 use uucore::entries::{self, Group, Locate, Passwd};
 use uucore::error::UResult;
 use uucore::error::{set_exit_code, USimpleError};
-use uucore::format_usage;
 pub use uucore::libc;
 use uucore::libc::{getlogin, uid_t};
+use uucore::line_ending::LineEnding;
 use uucore::process::{getegid, geteuid, getgid, getuid};
+use uucore::{format_usage, help_about, help_section, help_usage, show_error};
 
 macro_rules! cstr2cow {
     ($v:expr) => {
@@ -56,9 +51,9 @@ macro_rules! cstr2cow {
     };
 }
 
-static ABOUT: &str = "Print user and group information for each specified USER,
-or (when USER omitted) for the current user.";
-const USAGE: &str = "{} [OPTION]... [USER]...";
+const ABOUT: &str = help_about!("id.md");
+const USAGE: &str = help_usage!("id.md");
+const AFTER_HELP: &str = help_section!("after help", "id.md");
 
 #[cfg(not(feature = "selinux"))]
 static CONTEXT_HELP_TEXT: &str = "print only the security context of the process (not enabled)";
@@ -79,17 +74,6 @@ mod options {
     pub const ARG_USERS: &str = "USER";
 }
 
-fn get_description() -> String {
-    String::from(
-        "The id utility displays the user and group names and numeric IDs, of the \
-                      calling process, to the standard output. If the real and effective IDs are \
-                      different, both are displayed, otherwise only the real ID is displayed.\n\n\
-                      If a user (login name or user ID) is specified, the user and group IDs of \
-                      that user are displayed. In this case, the real and effective IDs are \
-                      assumed to be the same.",
-    )
-}
-
 struct Ids {
     uid: u32,  // user id
     gid: u32,  // group id
@@ -126,9 +110,7 @@ struct State {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let after_help = get_description();
-
-    let matches = uu_app().after_help(&after_help[..]).get_matches_from(args);
+    let matches = uu_app().after_help(AFTER_HELP).try_get_matches_from(args)?;
 
     let users: Vec<String> = matches
         .get_many::<String>(options::ARG_USERS)
@@ -136,13 +118,13 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         .unwrap_or_default();
 
     let mut state = State {
-        nflag: matches.contains_id(options::OPT_NAME),
-        uflag: matches.contains_id(options::OPT_EFFECTIVE_USER),
-        gflag: matches.contains_id(options::OPT_GROUP),
-        gsflag: matches.contains_id(options::OPT_GROUPS),
-        rflag: matches.contains_id(options::OPT_REAL_ID),
-        zflag: matches.contains_id(options::OPT_ZERO),
-        cflag: matches.contains_id(options::OPT_CONTEXT),
+        nflag: matches.get_flag(options::OPT_NAME),
+        uflag: matches.get_flag(options::OPT_EFFECTIVE_USER),
+        gflag: matches.get_flag(options::OPT_GROUP),
+        gsflag: matches.get_flag(options::OPT_GROUPS),
+        rflag: matches.get_flag(options::OPT_REAL_ID),
+        zflag: matches.get_flag(options::OPT_ZERO),
+        cflag: matches.get_flag(options::OPT_CONTEXT),
 
         selinux_supported: {
             #[cfg(feature = "selinux")]
@@ -190,13 +172,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             " ".to_string()
         }
     };
-    let line_ending = {
-        if state.zflag {
-            '\0'
-        } else {
-            '\n'
-        }
-    };
+    let line_ending = LineEnding::from_zero_flag(state.zflag);
 
     if state.cflag {
         if state.selinux_supported {
@@ -219,9 +195,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     }
 
     for i in 0..=users.len() {
-        let possible_pw = if !state.user_specified {
-            None
-        } else {
+        let possible_pw = if state.user_specified {
             match Passwd::locate(users[i].as_str()) {
                 Ok(p) => Some(p),
                 Err(_) => {
@@ -234,20 +208,22 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                     }
                 }
             }
+        } else {
+            None
         };
 
         // GNU's `id` does not support the flags: -p/-P/-A.
-        if matches.contains_id(options::OPT_PASSWORD) {
+        if matches.get_flag(options::OPT_PASSWORD) {
             // BSD's `id` ignores all but the first specified user
             pline(possible_pw.as_ref().map(|v| v.uid));
             return Ok(());
         };
-        if matches.contains_id(options::OPT_HUMAN_READABLE) {
+        if matches.get_flag(options::OPT_HUMAN_READABLE) {
             // BSD's `id` ignores all but the first specified user
             pretty(possible_pw);
             return Ok(());
         }
-        if matches.contains_id(options::OPT_AUDIT) {
+        if matches.get_flag(options::OPT_AUDIT) {
             // BSD's `id` ignores specified users
             auditid();
             return Ok(());
@@ -329,9 +305,9 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         }
 
         if default_format {
-            id_print(&mut state, &groups);
+            id_print(&state, &groups);
         }
-        print!("{}", line_ending);
+        print!("{line_ending}");
 
         if i + 1 >= users.len() {
             break;
@@ -341,7 +317,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     Ok(())
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
@@ -350,7 +326,7 @@ pub fn uu_app<'a>() -> Command<'a> {
         .arg(
             Arg::new(options::OPT_AUDIT)
                 .short('A')
-                .conflicts_with_all(&[
+                .conflicts_with_all([
                     options::OPT_GROUP,
                     options::OPT_EFFECTIVE_USER,
                     options::OPT_HUMAN_READABLE,
@@ -361,27 +337,30 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .help(
                     "Display the process audit user ID and other process audit properties,\n\
                       which requires privilege (not available on Linux).",
-                ),
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OPT_EFFECTIVE_USER)
                 .short('u')
                 .long(options::OPT_EFFECTIVE_USER)
                 .conflicts_with(options::OPT_GROUP)
-                .help("Display only the effective user ID as a number."),
+                .help("Display only the effective user ID as a number.")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OPT_GROUP)
                 .short('g')
                 .long(options::OPT_GROUP)
                 .conflicts_with(options::OPT_EFFECTIVE_USER)
-                .help("Display only the effective group ID as a number"),
+                .help("Display only the effective group ID as a number")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OPT_GROUPS)
                 .short('G')
                 .long(options::OPT_GROUPS)
-                .conflicts_with_all(&[
+                .conflicts_with_all([
                     options::OPT_GROUP,
                     options::OPT_EFFECTIVE_USER,
                     options::OPT_CONTEXT,
@@ -392,12 +371,14 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .help(
                     "Display only the different group IDs as white-space separated numbers, \
                       in no particular order.",
-                ),
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OPT_HUMAN_READABLE)
                 .short('p')
-                .help("Make the output human-readable. Each display is on a separate line."),
+                .help("Make the output human-readable. Each display is on a separate line.")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OPT_NAME)
@@ -407,12 +388,14 @@ pub fn uu_app<'a>() -> Command<'a> {
                     "Display the name of the user or group ID for the -G, -g and -u options \
                       instead of the number.\nIf any of the ID numbers cannot be mapped into \
                       names, the number will be displayed as usual.",
-                ),
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OPT_PASSWORD)
                 .short('P')
-                .help("Display the id as a password file entry."),
+                .help("Display the id as a password file entry.")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OPT_REAL_ID)
@@ -421,7 +404,8 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .help(
                     "Display the real ID for the -G, -g and -u options instead of \
                       the effective ID.",
-                ),
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OPT_ZERO)
@@ -430,19 +414,20 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .help(
                     "delimit entries with NUL characters, not whitespace;\n\
                       not permitted in default format",
-                ),
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::OPT_CONTEXT)
                 .short('Z')
                 .long(options::OPT_CONTEXT)
-                .conflicts_with_all(&[options::OPT_GROUP, options::OPT_EFFECTIVE_USER])
-                .help(CONTEXT_HELP_TEXT),
+                .conflicts_with_all([options::OPT_GROUP, options::OPT_EFFECTIVE_USER])
+                .help(CONTEXT_HELP_TEXT)
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::ARG_USERS)
-                .multiple_occurrences(true)
-                .takes_value(true)
+                .action(ArgAction::Append)
                 .value_name(options::ARG_USERS)
                 .value_hint(clap::ValueHint::Username),
         )
@@ -464,11 +449,11 @@ fn pretty(possible_pw: Option<Passwd>) {
         let rid = getuid();
         if let Ok(p) = Passwd::locate(rid) {
             if login == p.name {
-                println!("login\t{}", login);
+                println!("login\t{login}");
             }
             println!("uid\t{}", p.name);
         } else {
-            println!("uid\t{}", rid);
+            println!("uid\t{rid}");
         }
 
         let eid = getegid();
@@ -476,7 +461,7 @@ fn pretty(possible_pw: Option<Passwd>) {
             if let Ok(p) = Passwd::locate(eid) {
                 println!("euid\t{}", p.name);
             } else {
-                println!("euid\t{}", eid);
+                println!("euid\t{eid}");
             }
         }
 
@@ -485,7 +470,7 @@ fn pretty(possible_pw: Option<Passwd>) {
             if let Ok(g) = Group::locate(rid) {
                 println!("euid\t{}", g.name);
             } else {
-                println!("euid\t{}", rid);
+                println!("euid\t{rid}");
             }
         }
 
@@ -562,7 +547,7 @@ fn auditid() {
     println!("asid={}", auditinfo.ai_asid);
 }
 
-fn id_print(state: &mut State, groups: &[u32]) {
+fn id_print(state: &State, groups: &[u32]) {
     let uid = state.ids.as_ref().unwrap().uid;
     let gid = state.ids.as_ref().unwrap().gid;
     let euid = state.ids.as_ref().unwrap().euid;
