@@ -1,9 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) Konstantin Pospelov <kupospelov@gmail.com>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 // spell-checker:ignore (ToDO) autoformat FILENUM whitespaces pairable unpairable nocheck
 
@@ -22,6 +20,7 @@ use std::num::IntErrorKind;
 use std::os::unix::ffi::OsStrExt;
 use uucore::display::Quotable;
 use uucore::error::{set_exit_code, UError, UResult, USimpleError};
+use uucore::line_ending::LineEnding;
 use uucore::{crash, crash_if_err, format_usage, help_about, help_usage};
 
 const ABOUT: &str = help_about!("join.md");
@@ -62,13 +61,6 @@ enum FileNum {
     File2,
 }
 
-#[repr(u8)]
-#[derive(Copy, Clone)]
-enum LineEnding {
-    Nul = 0,
-    Newline = b'\n',
-}
-
 #[derive(Copy, Clone, PartialEq)]
 enum Sep {
     Char(u8),
@@ -600,6 +592,7 @@ impl<'a> State<'a> {
 }
 
 #[uucore::main]
+#[allow(clippy::cognitive_complexity)]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let matches = uu_app().try_get_matches_from(args)?;
 
@@ -682,9 +675,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         settings.headers = true;
     }
 
-    if matches.get_flag("z") {
-        settings.line_ending = LineEnding::Nul;
-    }
+    settings.line_ending = LineEnding::from_zero_flag(matches.get_flag("z"));
 
     let file1 = matches.get_one::<String>("file1").unwrap();
     let file2 = matches.get_one::<String>("file2").unwrap();
