@@ -1,3 +1,7 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 // spell-checker:ignore formatteriteminfo blocksize thisblock
 
 use std::cmp;
@@ -54,7 +58,7 @@ impl OutputInfo {
         line_bytes: usize,
         formats: &[ParsedFormatterItemInfo],
         output_duplicates: bool,
-    ) -> OutputInfo {
+    ) -> Self {
         let byte_size_block = formats.iter().fold(1, |max, next| {
             cmp::max(max, next.formatter_item_info.byte_size)
         });
@@ -68,9 +72,9 @@ impl OutputInfo {
         let print_width_line = print_width_block * (line_bytes / byte_size_block);
 
         let spaced_formatters =
-            OutputInfo::create_spaced_formatter_info(formats, byte_size_block, print_width_block);
+            Self::create_spaced_formatter_info(formats, byte_size_block, print_width_block);
 
-        OutputInfo {
+        Self {
             byte_size_line: line_bytes,
             print_width_line,
             byte_size_block,
@@ -90,7 +94,7 @@ impl OutputInfo {
             .map(|f| SpacedFormatterItemInfo {
                 formatter_item_info: f.formatter_item_info,
                 add_ascii_dump: f.add_ascii_dump,
-                spacing: OutputInfo::calculate_alignment(f, byte_size_block, print_width_block),
+                spacing: Self::calculate_alignment(f, byte_size_block, print_width_block),
             })
             .collect()
     }
@@ -139,7 +143,7 @@ impl OutputInfo {
     /// ```
     ///
     /// This algorithm assumes the size of all types is a power of 2 (1, 2, 4, 8, 16, ...)
-    /// Increase MAX_BYTES_PER_UNIT to allow larger types.
+    /// Increase `MAX_BYTES_PER_UNIT` to allow larger types.
     fn calculate_alignment(
         sf: &dyn TypeSizeInfo,
         byte_size_block: usize,
