@@ -1,14 +1,13 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) Alex Lyon <arcterus@mail.com>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 // spell-checker:ignore (path) eacces
 
-use clap::{crate_version, parser::ValueSource, Arg, ArgAction, Command};
+use clap::{builder::ValueParser, crate_version, parser::ValueSource, Arg, ArgAction, Command};
 use std::collections::VecDeque;
+use std::ffi::{OsStr, OsString};
 use std::fs::{self, File, Metadata};
 use std::io::ErrorKind;
 use std::ops::BitOr;
@@ -59,9 +58,9 @@ static ARG_FILES: &str = "files";
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let matches = uu_app().after_help(AFTER_HELP).try_get_matches_from(args)?;
 
-    let files: Vec<String> = matches
-        .get_many::<String>(ARG_FILES)
-        .map(|v| v.map(ToString::to_string).collect())
+    let files: Vec<&OsStr> = matches
+        .get_many::<OsString>(ARG_FILES)
+        .map(|v| v.map(OsString::as_os_str).collect())
         .unwrap_or_default();
 
     let force_flag = matches.get_flag(OPT_FORCE);
@@ -114,11 +113,20 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             verbose: matches.get_flag(OPT_VERBOSE),
         };
         if options.interactive == InteractiveMode::Once && (options.recursive || files.len() > 3) {
-            let msg = if options.recursive {
-                "Remove all arguments recursively?"
-            } else {
-                "Remove all arguments?"
-            };
+            let msg: String = format!(
+                "remove {} {}{}",
+                files.len(),
+                if files.len() > 1 {
+                    "arguments"
+                } else {
+                    "argument"
+                },
+                if options.recursive {
+                    " recursively?"
+                } else {
+                    "?"
+                }
+            );
             if !prompt_yes!("{}", msg) {
                 return Ok(());
             }
@@ -168,6 +176,9 @@ pub fn uu_app() -> Command {
                     prompts always",
                 )
                 .value_name("WHEN")
+                .num_args(0..=1)
+                .require_equals(true)
+                .default_missing_value("always")
                 .overrides_with_all([OPT_PROMPT, OPT_PROMPT_MORE]),
         )
         .arg(
@@ -231,13 +242,14 @@ pub fn uu_app() -> Command {
         .arg(
             Arg::new(ARG_FILES)
                 .action(ArgAction::Append)
+                .value_parser(ValueParser::os_string())
                 .num_args(1..)
                 .value_hint(clap::ValueHint::AnyPath),
         )
 }
 
 // TODO: implement one-file-system (this may get partially implemented in walkdir)
-fn remove(files: &[String], options: &Options) -> bool {
+fn remove(files: &[&OsStr], options: &Options) -> bool {
     let mut had_err = false;
 
     for filename in files {
@@ -274,6 +286,7 @@ fn remove(files: &[String], options: &Options) -> bool {
     had_err
 }
 
+#[allow(clippy::cognitive_complexity)]
 fn handle_dir(path: &Path, options: &Options) -> bool {
     let mut had_err = false;
 
@@ -355,7 +368,7 @@ fn handle_dir(path: &Path, options: &Options) -> bool {
 }
 
 fn remove_dir(path: &Path, options: &Options) -> bool {
-    if prompt_file(path, options, true) {
+    if prompt_dir(path, options) {
         if let Ok(mut read_dir) = fs::read_dir(path) {
             if options.dir || options.recursive {
                 if read_dir.next().is_none() {
@@ -400,7 +413,7 @@ fn remove_dir(path: &Path, options: &Options) -> bool {
 }
 
 fn remove_file(path: &Path, options: &Options) -> bool {
-    if prompt_file(path, options, false) {
+    if prompt_file(path, options) {
         match fs::remove_file(path) {
             Ok(_) => {
                 if options.verbose {
@@ -422,7 +435,23 @@ fn remove_file(path: &Path, options: &Options) -> bool {
     false
 }
 
-fn prompt_file(path: &Path, options: &Options, is_dir: bool) -> bool {
+fn prompt_dir(path: &Path, options: &Options) -> bool {
+    // If interactive is Never we never want to send prompts
+    if options.interactive == InteractiveMode::Never {
+        return true;
+    }
+
+    // We can't use metadata.permissions.readonly for directories because it only works on files
+    // So we have to handle whether a directory is writable manually
+    if let Ok(metadata) = fs::metadata(path) {
+        handle_writable_directory(path, options, &metadata)
+    } else {
+        true
+    }
+}
+
+#[allow(clippy::cognitive_complexity)]
+fn prompt_file(path: &Path, options: &Options) -> bool {
     // If interactive is Never we never want to send prompts
     if options.interactive == InteractiveMode::Never {
         return true;
@@ -435,58 +464,45 @@ fn prompt_file(path: &Path, options: &Options, is_dir: bool) -> bool {
             }
         }
     }
-    if is_dir {
-        // We can't use metadata.permissions.readonly for directories because it only works on files
-        // So we have to handle whether a directory is writable on not manually
-        if let Ok(metadata) = fs::metadata(path) {
-            handle_writable_directory(path, options, &metadata)
-        } else {
-            true
-        }
-    } else {
-        // File::open(path) doesn't open the file in write mode so we need to use file options to open it in also write mode to check if it can written too
-        match File::options().read(true).write(true).open(path) {
-            Ok(file) => {
-                if let Ok(metadata) = file.metadata() {
-                    if metadata.permissions().readonly() {
-                        if metadata.len() == 0 {
-                            prompt_yes!(
-                                "remove write-protected regular empty file {}?",
-                                path.quote()
-                            )
-                        } else {
-                            prompt_yes!("remove write-protected regular file {}?", path.quote())
-                        }
-                    } else if options.interactive == InteractiveMode::Always {
-                        if metadata.len() == 0 {
-                            prompt_yes!("remove regular empty file {}?", path.quote())
-                        } else {
-                            prompt_yes!("remove file {}?", path.quote())
-                        }
-                    } else {
-                        true
-                    }
-                } else {
-                    true
-                }
-            }
-            Err(err) => {
-                if err.kind() == ErrorKind::PermissionDenied {
-                    if let Ok(metadata) = fs::metadata(path) {
-                        if metadata.len() == 0 {
-                            prompt_yes!(
-                                "remove write-protected regular empty file {}?",
-                                path.quote()
-                            )
-                        } else {
-                            prompt_yes!("remove write-protected regular file {}?", path.quote())
-                        }
+    // File::open(path) doesn't open the file in write mode so we need to use file options to open it in also write mode to check if it can written too
+    match File::options().read(true).write(true).open(path) {
+        Ok(file) => {
+            if let Ok(metadata) = file.metadata() {
+                if metadata.permissions().readonly() {
+                    if metadata.len() == 0 {
+                        prompt_yes!(
+                            "remove write-protected regular empty file {}?",
+                            path.quote()
+                        )
                     } else {
                         prompt_yes!("remove write-protected regular file {}?", path.quote())
                     }
+                } else if options.interactive == InteractiveMode::Always {
+                    if metadata.len() == 0 {
+                        prompt_yes!("remove regular empty file {}?", path.quote())
+                    } else {
+                        prompt_yes!("remove file {}?", path.quote())
+                    }
                 } else {
                     true
                 }
+            } else {
+                true
+            }
+        }
+        Err(err) => {
+            if err.kind() == ErrorKind::PermissionDenied {
+                match fs::metadata(path) {
+                    Ok(metadata) if metadata.len() == 0 => {
+                        prompt_yes!(
+                            "remove write-protected regular empty file {}?",
+                            path.quote()
+                        )
+                    }
+                    _ => prompt_yes!("remove write-protected regular file {}?", path.quote()),
+                }
+            } else {
+                true
             }
         }
     }
