@@ -1,9 +1,12 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 // spell-checker:ignore extendedbigdecimal extendedbigint bigdecimal numberparse
 //! Parsing numbers for use in `seq`.
 //!
 //! This module provides an implementation of [`FromStr`] for the
 //! [`PreciseNumber`] struct.
-use std::convert::TryInto;
 use std::str::FromStr;
 
 use bigdecimal::BigDecimal;
@@ -18,7 +21,7 @@ use crate::number::Number;
 use crate::number::PreciseNumber;
 
 /// An error returned when parsing a number fails.
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Eq)]
 pub enum ParseNumberError {
     Float,
     Nan,
