@@ -1,9 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) Michael Debertol <michael.debertol..AT..gmail.com>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 //! Sort big files by using auxiliary files for storing intermediate chunks.
 //!
@@ -84,11 +82,7 @@ fn reader_writer<
     output: Output,
     tmp_dir: &mut TmpDirWrapper,
 ) -> UResult<()> {
-    let separator = if settings.zero_terminated {
-        b'\0'
-    } else {
-        b'\n'
-    };
+    let separator = settings.line_ending.into();
 
     // Heuristically chosen: Dividing by 10 seems to keep our memory usage roughly
     // around settings.buffer_size as a whole.
@@ -158,7 +152,7 @@ fn reader_writer<
 /// The function that is executed on the sorter thread.
 fn sorter(receiver: &Receiver<Chunk>, sender: &SyncSender<Chunk>, settings: &GlobalSettings) {
     while let Ok(mut payload) = receiver.recv() {
-        payload.with_contents_mut(|contents| {
+        payload.with_dependent_mut(|_, contents| {
             sort_by(&mut contents.lines, settings, &contents.line_data);
         });
         if sender.send(payload).is_err() {
