@@ -1,18 +1,17 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 // spell-checker:ignore (ToDO) kqueue Signum fundu
 
 use crate::paths::Input;
 use crate::{parse, platform, Quotable};
-use clap::crate_version;
+use clap::{crate_version, value_parser};
 use clap::{Arg, ArgAction, ArgMatches, Command};
-use fundu::DurationParser;
+use fundu::{DurationParser, SaturatingInto};
 use is_terminal::IsTerminal;
 use same_file::Handle;
-use std::collections::VecDeque;
 use std::ffi::OsString;
 use std::time::Duration;
 use uucore::error::{UResult, USimpleError, UUsageError};
@@ -141,7 +140,8 @@ pub struct Settings {
     pub use_polling: bool,
     pub verbose: bool,
     pub presume_input_pipe: bool,
-    pub inputs: VecDeque<Input>,
+    /// `FILE(s)` positional arguments
+    pub inputs: Vec<Input>,
 }
 
 impl Default for Settings {
@@ -149,21 +149,21 @@ impl Default for Settings {
         Self {
             max_unchanged_stats: 5,
             sleep_sec: Duration::from_secs_f32(1.0),
-            follow: Default::default(),
-            mode: Default::default(),
+            follow: Option::default(),
+            mode: FilterMode::default(),
             pid: Default::default(),
             retry: Default::default(),
             use_polling: Default::default(),
             verbose: Default::default(),
             presume_input_pipe: Default::default(),
-            inputs: Default::default(),
+            inputs: Vec::default(),
         }
     }
 }
 
 impl Settings {
     pub fn from_obsolete_args(args: &parse::ObsoleteArgs, name: Option<&OsString>) -> Self {
-        let mut settings: Self = Default::default();
+        let mut settings = Self::default();
         if args.follow {
             settings.follow = if name.is_some() {
                 Some(FollowMode::Name)
@@ -173,11 +173,11 @@ impl Settings {
         }
         settings.mode = FilterMode::from_obsolete_args(args);
         let input = if let Some(name) = name {
-            Input::from(&name)
+            Input::from(name)
         } else {
             Input::default()
         };
-        settings.inputs.push_back(input);
+        settings.inputs.push(input);
         settings
     }
 
@@ -235,12 +235,15 @@ impl Settings {
             //   `DURATION::MAX` or `infinity` was given
             // * not applied here but it supports customizable time units and provides better error
             //   messages
-            settings.sleep_sec =
-                DurationParser::without_time_units()
-                    .parse(source)
-                    .map_err(|_| {
-                        UUsageError::new(1, format!("invalid number of seconds: '{source}'"))
-                    })?;
+            settings.sleep_sec = match DurationParser::without_time_units().parse(source) {
+                Ok(duration) => SaturatingInto::<std::time::Duration>::saturating_into(duration),
+                Err(_) => {
+                    return Err(UUsageError::new(
+                        1,
+                        format!("invalid number of seconds: '{source}'"),
+                    ))
+                }
+            }
         }
 
         if let Some(s) = matches.get_one::<String>(options::MAX_UNCHANGED_STATS) {
@@ -282,19 +285,13 @@ impl Settings {
             }
         }
 
-        let mut inputs: VecDeque<Input> = matches
-            .get_many::<String>(options::ARG_FILES)
-            .map(|v| v.map(|string| Input::from(&string)).collect())
-            .unwrap_or_default();
+        settings.inputs = matches
+            .get_many::<OsString>(options::ARG_FILES)
+            .map(|v| v.map(Input::from).collect())
+            .unwrap_or_else(|| vec![Input::default()]);
 
-        // apply default and add '-' to inputs if none is present
-        if inputs.is_empty() {
-            inputs.push_front(Input::default());
-        }
-
-        settings.verbose = inputs.len() > 1 && !matches.get_flag(options::verbosity::QUIET);
-
-        settings.inputs = inputs;
+        settings.verbose =
+            settings.inputs.len() > 1 && !matches.get_flag(options::verbosity::QUIET);
 
         Ok(settings)
     }
@@ -590,6 +587,7 @@ pub fn uu_app() -> Command {
             Arg::new(options::ARG_FILES)
                 .action(ArgAction::Append)
                 .num_args(1..)
+                .value_parser(value_parser!(OsString))
                 .value_hint(clap::ValueHint::FilePath),
         )
 }
