@@ -1,5 +1,11 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 // features ~ feature-gated modules (core/bundler file)
 
+#[cfg(feature = "backup-control")]
+pub mod backup_control;
 #[cfg(feature = "encoding")]
 pub mod encoding;
 #[cfg(feature = "fs")]
@@ -10,10 +16,16 @@ pub mod fsext;
 pub mod lines;
 #[cfg(feature = "memo")]
 pub mod memo;
+#[cfg(feature = "ranges")]
+pub mod ranges;
 #[cfg(feature = "ringbuffer")]
 pub mod ringbuffer;
+#[cfg(feature = "sum")]
+pub mod sum;
 #[cfg(feature = "memo")]
 mod tokenize;
+#[cfg(feature = "update-control")]
+pub mod update_control;
 
 // * (platform-specific) feature-gated modules
 // ** non-windows (i.e. Unix + Fuchsia)
@@ -34,6 +46,7 @@ pub mod process;
 pub mod signals;
 #[cfg(all(
     unix,
+    not(target_os = "android"),
     not(target_os = "fuchsia"),
     not(target_os = "redox"),
     not(target_env = "musl"),
