@@ -1,6 +1,10 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 // spell-checker:ignore (words) araba merci
 
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 
 #[test]
 fn test_default() {
@@ -176,7 +180,7 @@ fn test_disable_escapes() {
         .arg("-E")
         .arg(input_str)
         .succeeds()
-        .stdout_only(format!("{}\n", input_str));
+        .stdout_only(format!("{input_str}\n"));
 }
 
 #[test]
