@@ -1,4 +1,8 @@
-use crate::common::util::*;
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
+use crate::common::util::TestScenario;
 
 #[test]
 fn basic_literal() {
@@ -446,6 +450,14 @@ fn sub_any_specifiers_after_period() {
         .stdout_only("3");
 }
 
+#[test]
+fn unspecified_left_justify_is_1_width() {
+    new_ucmd!()
+        .args(&["%-o"]) //spell-checker:disable-line
+        .succeeds()
+        .stdout_only("0");
+}
+
 #[test]
 fn sub_any_specifiers_after_second_param() {
     new_ucmd!()
